using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using ZedGraph;

namespace StatisticalAnalyzer
{
	/// <summary>
	/// Summary description for MeanGraphForAllRounds.
	/// </summary>
	public class MeanGraphUnderDifferentConditions : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private GraphPane myPane;		
		private ArrayList receivedList;
		Color[] LineColor = 
		{Color.Red, Color.Blue, Color.YellowGreen, Color.Orange, Color.Green};	
		SymbolType[] Symbol = {SymbolType.Circle, SymbolType.Diamond, SymbolType.Star, 
								  SymbolType.Triangle, SymbolType.Square};

		public MeanGraphUnderDifferentConditions()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			receivedList = new ArrayList();
			
		}

		public MeanGraphUnderDifferentConditions(ArrayList p_GraphDataList)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			try
			{
				receivedList = p_GraphDataList;				
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in constructor of MeanGraphForAllRounds():"+e.Message);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// MeanGraphForAllRounds
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(920, 589);
			this.Name = "MeanGraphForAllRounds";
			this.Text = "MeanGraphForAllRounds";
			this.Load += new System.EventHandler(this.MeanGraphForAllRounds_Load);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.MeanGraphForAllRounds_Paint);

		}
		#endregion


		private double getValue(GraphData p_myGD, string p_key)
		{
			double D = 0;
			try
			{
				IDictionaryEnumerator ide = p_myGD.XParam_Interval.GetEnumerator();
				while(ide.MoveNext())
				{
					if(ide.Key.ToString().Equals(p_key))
					{
						D = double.Parse(ide.Value.ToString());  
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getValue(): "+e.Message);
			}
			return D;
		}
		private double[] getXs(GraphData p_myGD)
		{
			int length = p_myGD.XParam_Interval.Count;
			double[] doubleXParam = new double[length];
			try
			{
				for(int i = 1; i<=length; i++)
				{
					doubleXParam[i-1] = getValue(p_myGD,i.ToString());
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getX(): "+e.Message);
			}
			return doubleXParam;
		}

		private double[] convertToDoubleArray(ArrayList p_al)
		{
			double[] doubleAL = new double[p_al.Count];
			try
			{
				for(int i =0; i< p_al.Count; i++)
				{
					doubleAL[i] = double.Parse(p_al[i].ToString());
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in convertToDoubleArray(): "+e.Message);
			}
			return doubleAL;
		}
		private double[] getDoubleYValue(GraphData p_myGD, string p_round)
		{
			double[] doubleYValues = new double[p_myGD.XParam_Interval.Count];
			try
			{
				IDictionaryEnumerator ide = p_myGD.DataForEachRound.GetEnumerator();
				while(ide.MoveNext())
				{
					if(ide.Key.ToString().Equals(p_round))
					{
						ArrayList temp = (ArrayList) ide.Value;
						doubleYValues = convertToDoubleArray(temp);
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getDoubleYValue(): "+e.Message);
			}
			return doubleYValues;
		}
		private double[] getDoubleYs(double[] p_doubleYMean, double[] p_doubleYValue)
		{			
			try
			{
				for(int i= 0; i<p_doubleYMean.Length; i++)
				{
					p_doubleYMean[i] += p_doubleYValue[i];
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getDoubleYMean(): "+e.Message);
			}
			return p_doubleYMean;
		}
		private void showDoubleYValue(double[] p_doubleYValue)
		{
			string inside = "";
			for(int i = 0; i<p_doubleYValue.Length; i++)
			{
				inside += p_doubleYValue[i].ToString()+" ";
			}
			MessageBox.Show(inside);
		}

		private double[] getAvgDoubleYs(GraphData p_myGD, double[] doubleYs)
		{
			double[] avgDoubleYs = new double[doubleYs.Length];
			try
			{
				double numOfRound = double.Parse(p_myGD.DataForEachRound.Count.ToString());
				for(int i =0; i<doubleYs.Length; i++)
				{
					avgDoubleYs[i] = (doubleYs[i]/numOfRound);
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getAvgDoubleYs(): "+e.Message);
			}
			return avgDoubleYs;
		}
		private double[] getMeanY(GraphData p_myGD)
		{
			
			double[] doubleYs = new double[p_myGD.XParam_Interval.Count];
			double[] doubleYMean = new double[p_myGD.XParam_Interval.Count];
			try
			{
				for(int i =1; i<= p_myGD.DataForEachRound.Count; i++)
				{
					double[] doubleYValue = getDoubleYValue(p_myGD, i.ToString());
					//showDoubleYValue(doubleYValue);
					doubleYs = getDoubleYs(doubleYs, doubleYValue);
				}
				doubleYMean = getAvgDoubleYs(p_myGD,doubleYs);				
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getMeanY(): "+e.Message);
			}
			return doubleYMean;
		}

		private void showDoubleArray(double[] p_al)
		{
			string inside = "";
			for(int i =0; i<p_al.Length; i++)
			{
				inside += p_al[i].ToString()+" ";
			}
			MessageBox.Show(inside);
		}
		private void ShowGraph()
		{

			SavedGraphElements FirstSGE = (SavedGraphElements)receivedList[0];
			GraphData myGD = FirstSGE.GD;
			
			myPane = new GraphPane(new Rectangle(20,20, this.Width-50, this.Height-110), 
				"Mean Graph Under Different Conditions", myGD.x_parameter, myGD.y_parameter);

			double[] x = getXs(myGD);
							
			for(int i = 0; i<=receivedList.Count; i++)
			{			
				SavedGraphElements SGE = (SavedGraphElements)receivedList[i];
				string curveName = SGE.strCondition;
				GraphData GD = SGE.GD;
				double[] y = getMeanY(GD);
				CurveItem myCurve1 = myPane.AddCurve(curveName, x, y, 
					LineColor[i-1], Symbol[i-1]);
			}
			myPane.AxisChange();				
		}
		private void MeanGraphForAllRounds_Load(object sender, System.EventArgs e)
		{
			ShowGraph();
		}

		private void MeanGraphForAllRounds_Paint(object sender, PaintEventArgs e)
		{
			myPane.Draw(e.Graphics);
		}
	}
}
