/*Author: Jin-Hee Cho 
* Project Name: Wireless Ad Hoc Messenger Using Pocket PCs
* Purpose: Thesis for Master Science in Computer Science 
* at Virginia Polytechnic State University
* Funded by Microsoft
* Date Updated: 7/29/2004
* StatisticalAnalyzer is the application running on the latop to collect result files 
* from each Pocket PC and generate a result graph according to selected x and y parameter.
*/


using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using System.Net.Sockets;
using System.Net;
using System.IO;
using System.Text;
using System.Xml;

namespace StatisticalAnalyzer
{
	/// <summary>
	/// Summary description for StatisticalAnalyzer.
	/// </summary>
	public class StatisticalAnalyzer : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Panel panel_KeyParameter;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label4;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Button Exit_Btn;
		private System.Windows.Forms.TextBox Status_TB;
		private System.Windows.Forms.Button BasicInfo_Btn;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label NumOfRound_Btn;
		private System.Windows.Forms.TextBox NumOfRound_TB;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.ListBox listBox_Session;
		private System.Windows.Forms.ListBox listBox_Round;
		private System.Windows.Forms.Button Gen_Session_Round_Btn;
		private System.Windows.Forms.TextBox Available_Files_TB;
		private System.Windows.Forms.TextBox NumOfSession_TB;
		private System.Windows.Forms.Button DeleteFiles_Btn;		
		private System.Windows.Forms.Button AvailanleFiles_Btn;
		private System.Windows.Forms.Button DeleteFile_Btn;
		private System.Windows.Forms.TextBox MyIP;
		private System.Windows.Forms.Button GraphForEachRound_Btn;
		private System.Windows.Forms.Button GraphAllRounds_Btn;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.ListBox XParam_LB;
		private System.Windows.Forms.ListBox YParam_LB;
		private System.Windows.Forms.Button ResetForStatusTB_Btn;
		
		//***********************************************************************//
		//UdpServer
		//***********************************************************************//
		private Socket udpSocket;
		private Thread udpListeningThread;
		public string ReceivedData;
		public int ReceivedDataLength;
	
		//********************************************************************//
		//UdpClient
		private UdpClient udpClient;
		private bool udpServerIsAlive;

		//Other variables
		private string strMyIP;
		private string strMyID;		
		private string ResultsCollected;
		private int fileID;
		public Hashtable SenderBuffer;
		private ArrayList DataToSendBuffer;
		private string file_name ="";
		private Hashtable ReceiptTimerBuffer;
		private string selectedSession = "";
		private string selectedRound = "";
		private string selectedXParam = "";
		private string selectedYParam = "";	
		private ArrayList possibleFilesToBeReceived;
		private System.Windows.Forms.Button CompareGraph_Btn;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.TextBox Path_TB;
		private ArrayList SavedGraphList;
		
		/// <summary>
		/// Required designer variable.
		/// </summary>		

		public StatisticalAnalyzer()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			try
			{
				fileID  = -1;
				ResultsCollected ="";				
				udpServerIsAlive = false;
				SenderBuffer = new Hashtable();
				start_server();												
				DataToSendBuffer = new ArrayList();
				ReceiptTimerBuffer = new Hashtable();
				possibleFilesToBeReceived = new ArrayList();
				SavedGraphList = new ArrayList();
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in Constructor(): "+e.Message);
			}
		}

		private void resetAll()
		{
			DataToSendBuffer.Clear();
			ResultsCollected ="";			
			SenderBuffer.Clear();
			fileID = -1;		
			Status_TB.Text = "";						
			Available_Files_TB.Text = "";
			possibleFilesToBeReceived.Clear();
		}

		private void stop_server()
		{
			udpListeningThread.Abort();
			
		}
		private void start_server()
		{
			try
			{		
				//Starting the UDP Listening thread.
				udpListeningThread = new Thread(new ThreadStart(this.receivePacket));
				udpServerIsAlive = true;				
				udpListeningThread.Start();
			}
			catch (Exception e)
			{
				MessageBox.Show("start_server() exception occurred : "+e.Message);
				udpListeningThread.Abort();				
			}
		}
						
		/// <summary>
		/// Receive UDP Packets
		/// </summary>
		public void receivePacket()
		{			
			IPHostEntry localHostEntry;
		
			try
			{
				udpSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
				localHostEntry = Dns.GetHostByName(Dns.GetHostName());
				IPEndPoint localIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], UDPCONSTANTS.UDPPORT);
				EndPoint localEndEntry = (localIpEndPoint);
				udpSocket.Bind(localEndEntry);
				while (udpServerIsAlive)
				{
					Monitor.Enter(this);
					Byte[] received = new Byte[UDPCONSTANTS.MESSAGESIZE];
					//This is just a dummy endpoint that will be replaced after the call to ReceiveFrom.
					//In the ReceiveFrom call later on, dummyIpEndPoint will be passed by ref.
					IPEndPoint dummyIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], UDPCONSTANTS.UDPPORT);

					EndPoint remoteNotifyEP = (dummyIpEndPoint);

					int udpBytesReceived = udpSocket.ReceiveFrom(received, ref remoteNotifyEP);

					ReceivedDataLength=udpBytesReceived;								
					String dataReceivedUdp = System.Text.Encoding.ASCII.GetString(received,0,received.Length );
					ReceivedData=dataReceivedUdp.ToString();								
					HandleReceivedPacket();
					Monitor.Exit(this);		
				}		
					
			}	
			catch (Exception e)
			{
				MessageBox.Show("Exception occurred in receivedPacket() :"+e.Message);
			}
		}
		
		
		private int getMessageLength()
		{
			string msgLength= "";
			try
			{								
				for(int i =0 ; i<ReceivedData.Length; i++)
				{
					if(ReceivedData[i].Equals(' '))
					{
						break;
					}
					else
					{
						msgLength = msgLength+ReceivedData[i].ToString();
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getMessageLength(): "+e.Message);
			}
			return int.Parse(msgLength);
		}
		private FilePacket transformToFilePacket()
		{
			FilePacket fp = new FilePacket();
			try
			{
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(ReceivedData);
				ReceivedDataLength=0;//reset the received data's length
				XmlNode root = doc.DocumentElement; 
				XmlNodeList xNodes = root.ChildNodes;
			
				foreach(XmlNode childNode in xNodes)
				{
					
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.SENDERIP))
					{
						fp.strSenderIP = elem.FirstChild.Value;
					}
					else if(elem.Name.Equals(COMMONCONSTANTS.ID))
					{
						fp.strID = elem.FirstChild.Value;
					}
					else if(elem.Name.Equals(COMMONCONSTANTS.NUMOFTOTALPACKETS))
					{
						fp.strNumOfTotalPackets= elem.FirstChild.Value;
					}
					else if(elem.Name.Equals(COMMONCONSTANTS.FILECONTENT))
					{
						fp.strXMLFile =  elem.FirstChild.Value;
					}
					else if(elem.Name.Equals(COMMONCONSTANTS.TYPE))
					{
						fp.strType =  elem.FirstChild.Value;					
					}
				}	
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in transformToFilePacket(): "+e.Message);
			}
			return fp;
		}

		private void SetReceiptTimer(FilePacket p_fp)
		{
			try
			{
				MyTimer myTimer = new MyTimer(1000,true,this,p_fp.strID, p_fp);
				ReceiptTimerBuffer.Add(p_fp.strID, myTimer);
				myTimer.SetReceiptTimer();
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SetReceiptTimer(): "+e.Message);
			}
		}
		private void SendReceiptPacket(ref FilePacket p_fp)
		{
			try
			{
				string targetIP = p_fp.strSenderIP;
				FilePacket fp = new FilePacket();
				fp.strType = COMMONCONSTANTS.RECEIPT_PACKET;
				fp.strSenderIP = strMyIP;
				fp.strID = p_fp.strID;
				fp.strNumOfTotalPackets = COMMONCONSTANTS.NOT_AVAILABLE;
				fp.strXMLFile = COMMONCONSTANTS.NOT_AVAILABLE;

				XmlDocument doc = new XmlDocument();
				doc.LoadXml("<FilePacket></FilePacket>");

				XmlNode root = doc.DocumentElement;

				//1. Type
				XmlElement elem = doc.CreateElement(COMMONCONSTANTS.TYPE);
				elem.InnerText = fp.strType;
				root.AppendChild(elem);

				//2.ID
				elem = doc.CreateElement(COMMONCONSTANTS.ID);
				elem.InnerText = fp.strID;
				root.AppendChild(elem);

				//3. SenderIP
				elem = doc.CreateElement(COMMONCONSTANTS.SENDERIP);
				elem.InnerText = fp.strSenderIP;
				root.AppendChild(elem);

				//4. Number of Total Packets
				elem = doc.CreateElement(COMMONCONSTANTS.NUMOFTOTALPACKETS);
				elem.InnerText = fp.strNumOfTotalPackets;
				root.AppendChild(elem);

				//5. File content
				elem = doc.CreateElement(COMMONCONSTANTS.FILECONTENT);
				elem.InnerText = fp.strXMLFile;
				root.AppendChild(elem);

				string messageToSend = doc.InnerXml.ToString();
			
				bool sentSuccess = sendUDP(targetIP,messageToSend);
				//MessageBox.Show("sentSuccess Receipt Packet: "+sentSuccess);
 
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SendReceiptPacket(): "+e.Message);
			}
		}

		private int getSequenceID(string p_ID)
		{
			int SID = -1;
			try
			{
				for(int i = 0; i<p_ID.Length; i++)
				{
					if(p_ID[i].ToString().Equals(":"))
					{
						string sequenceID = p_ID.Substring(i+1);
						SID = int.Parse(sequenceID);						
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getSequenceID(): "+e.Message);
			}
			//MessageBox.Show("SID: "+SID);
			return SID;
		}



		private void writeToFile(FilePacket p_fp)
		{
			try
			{
				StreamWriter myWriter;
				int index = getSequenceID(p_fp.strID);
				int numOfTotalPackets = int.Parse(p_fp.strNumOfTotalPackets);			
				if(index == 0)
				{
					file_name = p_fp.strXMLFile;
				}
				else if(index == numOfTotalPackets)
				{
					ResultsCollected += p_fp.strXMLFile;
					try
					{
						string AFID = selectedSession+":"+selectedRound;																		
//						string path = "\\MyData\\Jin-Hee\\PocketPC2002Chat\\ResultsCollected\\"+
//							selectedSession+"\\"+selectedRound+"\\";
						string path = Path_TB.Text+selectedSession+"\\"+selectedRound+"\\";
						myWriter = new StreamWriter(@path+file_name,false,Encoding.ASCII);
						myWriter.Write(ResultsCollected);
						myWriter.Close();					
					}
					catch(Exception e)
					{
                        MessageBox.Show("Please select the Session and Round directory to store the received file. "+e.Message);
					}
				}
				else
				{
					ResultsCollected += p_fp.strXMLFile;			
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in writeToFile(): "+e.Message);
			}
		}
		private void HandleReceivedPacket()
		{
			
			try
			{
				if(ReceivedData.Length>0)
				{	
					
					FilePacket fp = transformToFilePacket();					
					if(fp.strType.Equals(COMMONCONSTANTS.RECEIPT_PACKET))
					{
						ReleaseReceiptTimer(fp.strID);
						SenderBuffer.Remove(fp.strID);
						DataToSendBuffer.RemoveAt(0);
						if((DataToSendBuffer.Count!=0) && (SenderBuffer.Count == 0))
						{
							FilePacket temp_fp = new FilePacket();
							temp_fp = (FilePacket)DataToSendBuffer[0];
							SaveInSenderBuffer(temp_fp);
							SendFilePacket(temp_fp);  
						}
						else if(DataToSendBuffer.Count==0)
						{
							Status_TB.Text += "\r\n"+"Completed to Send "+file_name+".";
						}
					}
					else//File Packet
					{
						//MessageBox.Show("Received file");
						int index = getSequenceID(fp.strID);
						SendReceiptPacket(ref fp);
						writeToFile(fp);
						
						if(index == int.Parse(fp.strNumOfTotalPackets))
						{						   							
							Status_TB.Text += file_name+" is received from "+fp.strSenderIP+"\r\n";
							ResultsCollected ="";
						}
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in HandleReceivedPacket(): "+e.Message);
			}
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		//UdpClient relate methods
		public bool sendUDP(string serverIPaddr, string p_message)
		{
			Byte[] receivedData = new Byte[UDPCONSTANTS.MESSAGESIZE];
			bool sentSuccess = false;
			try
			{
				IPAddress serverIP = IPAddress.Parse(serverIPaddr);
				udpClient = new UdpClient();
				udpClient.Connect(serverIP,UDPCONSTANTS.UDPPORT);
		
				Byte[] inputToBeSent = new Byte[UDPCONSTANTS.MESSAGESIZE];
		
				//Convert the "message" string to a byte array, and then send it to
				//the remote UDP server using udpClient.
				inputToBeSent = System.Text.Encoding.ASCII.GetBytes(p_message.ToCharArray());
				int nBytesSent = udpClient.Send(inputToBeSent, inputToBeSent.Length);
				sentSuccess = true;				
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception is occurred in sendUDP() : "+e.Message);
			}
			udpClient.Close();
			return sentSuccess;
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new StatisticalAnalyzer());
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.panel_KeyParameter = new System.Windows.Forms.Panel();
			this.CompareGraph_Btn = new System.Windows.Forms.Button();
			this.YParam_LB = new System.Windows.Forms.ListBox();
			this.XParam_LB = new System.Windows.Forms.ListBox();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.DeleteFiles_Btn = new System.Windows.Forms.Button();
			this.label4 = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.MyIP = new System.Windows.Forms.TextBox();
			this.Exit_Btn = new System.Windows.Forms.Button();
			this.Status_TB = new System.Windows.Forms.TextBox();
			this.BasicInfo_Btn = new System.Windows.Forms.Button();
			this.label6 = new System.Windows.Forms.Label();
			this.NumOfSession_TB = new System.Windows.Forms.TextBox();
			this.NumOfRound_Btn = new System.Windows.Forms.Label();
			this.NumOfRound_TB = new System.Windows.Forms.TextBox();
			this.label7 = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.listBox_Session = new System.Windows.Forms.ListBox();
			this.listBox_Round = new System.Windows.Forms.ListBox();
			this.Gen_Session_Round_Btn = new System.Windows.Forms.Button();
			this.Available_Files_TB = new System.Windows.Forms.TextBox();
			this.AvailanleFiles_Btn = new System.Windows.Forms.Button();
			this.DeleteFile_Btn = new System.Windows.Forms.Button();
			this.GraphForEachRound_Btn = new System.Windows.Forms.Button();
			this.GraphAllRounds_Btn = new System.Windows.Forms.Button();
			this.ResetForStatusTB_Btn = new System.Windows.Forms.Button();
			this.label5 = new System.Windows.Forms.Label();
			this.Path_TB = new System.Windows.Forms.TextBox();
			this.panel_KeyParameter.SuspendLayout();
			this.SuspendLayout();
			// 
			// panel_KeyParameter
			// 
			this.panel_KeyParameter.BackColor = System.Drawing.Color.Goldenrod;
			this.panel_KeyParameter.Controls.Add(this.Path_TB);
			this.panel_KeyParameter.Controls.Add(this.label5);
			this.panel_KeyParameter.Controls.Add(this.CompareGraph_Btn);
			this.panel_KeyParameter.Controls.Add(this.YParam_LB);
			this.panel_KeyParameter.Controls.Add(this.XParam_LB);
			this.panel_KeyParameter.Controls.Add(this.label3);
			this.panel_KeyParameter.Controls.Add(this.label2);
			this.panel_KeyParameter.Controls.Add(this.DeleteFiles_Btn);
			this.panel_KeyParameter.Controls.Add(this.label4);
			this.panel_KeyParameter.Controls.Add(this.label1);
			this.panel_KeyParameter.Controls.Add(this.MyIP);
			this.panel_KeyParameter.Controls.Add(this.Exit_Btn);
			this.panel_KeyParameter.Location = new System.Drawing.Point(-16, 0);
			this.panel_KeyParameter.Name = "panel_KeyParameter";
			this.panel_KeyParameter.Size = new System.Drawing.Size(304, 544);
			this.panel_KeyParameter.TabIndex = 0;

			// 
			// CompareGraph_Btn
			// 
			this.CompareGraph_Btn.Location = new System.Drawing.Point(32, 456);
			this.CompareGraph_Btn.Name = "CompareGraph_Btn";
			this.CompareGraph_Btn.Size = new System.Drawing.Size(264, 32);
			this.CompareGraph_Btn.TabIndex = 13;
			this.CompareGraph_Btn.Text = "Go to Compare Graphs";
			this.CompareGraph_Btn.Click += new System.EventHandler(this.CompareGraph_Btn_Click);
			// 
			// YParam_LB
			// 
			this.YParam_LB.ItemHeight = 16;
			this.YParam_LB.Location = new System.Drawing.Point(24, 328);
			this.YParam_LB.Name = "YParam_LB";
			this.YParam_LB.ScrollAlwaysVisible = true;
			this.YParam_LB.Size = new System.Drawing.Size(272, 116);
			this.YParam_LB.TabIndex = 12;
			this.YParam_LB.SelectedIndexChanged += new System.EventHandler(this.YParam_LB_SelectedIndexChanged);
			// 
			// XParam_LB
			// 
			this.XParam_LB.ItemHeight = 16;
			this.XParam_LB.Location = new System.Drawing.Point(24, 208);
			this.XParam_LB.Name = "XParam_LB";
			this.XParam_LB.ScrollAlwaysVisible = true;
			this.XParam_LB.Size = new System.Drawing.Size(272, 68);
			this.XParam_LB.TabIndex = 11;
			this.XParam_LB.SelectedIndexChanged += new System.EventHandler(this.XParam_LB_SelectedIndexChanged);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(32, 296);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(56, 24);
			this.label3.TabIndex = 10;
			this.label3.Text = "Y-axis :";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(32, 184);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(56, 16);
			this.label2.TabIndex = 9;
			this.label2.Text = "X-axis :";
			// 
			// DeleteFiles_Btn
			// 
			this.DeleteFiles_Btn.Location = new System.Drawing.Point(176, 496);
			this.DeleteFiles_Btn.Name = "DeleteFiles_Btn";
			this.DeleteFiles_Btn.Size = new System.Drawing.Size(112, 40);
			this.DeleteFiles_Btn.TabIndex = 8;
			this.DeleteFiles_Btn.Text = "Delete All Files";
			this.DeleteFiles_Btn.Click += new System.EventHandler(this.DeleteFiles_Btn_Click);
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(24, 16);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(56, 24);
			this.label4.TabIndex = 6;
			this.label4.Text = "My IP :";
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(24, 160);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(256, 16);
			this.label1.TabIndex = 0;
			this.label1.Text = "Select Key Parameters :";
			// 
			// MyIP
			// 
			this.MyIP.Location = new System.Drawing.Point(104, 16);
			this.MyIP.Name = "MyIP";
			this.MyIP.ReadOnly = true;
			this.MyIP.Size = new System.Drawing.Size(168, 23);
			this.MyIP.TabIndex = 1;
			this.MyIP.Text = "";
			// 
			// Exit_Btn
			// 
			this.Exit_Btn.Location = new System.Drawing.Point(32, 496);
			this.Exit_Btn.Name = "Exit_Btn";
			this.Exit_Btn.Size = new System.Drawing.Size(120, 40);
			this.Exit_Btn.TabIndex = 2;
			this.Exit_Btn.Text = "Exit";
			this.Exit_Btn.Click += new System.EventHandler(this.Exit_Btn_Click);
			// 
			// Status_TB
			// 
			this.Status_TB.Location = new System.Drawing.Point(320, 320);
			this.Status_TB.Multiline = true;
			this.Status_TB.Name = "Status_TB";
			this.Status_TB.ReadOnly = true;
			this.Status_TB.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.Status_TB.Size = new System.Drawing.Size(216, 168);
			this.Status_TB.TabIndex = 3;
			this.Status_TB.Text = "";
			// 
			// BasicInfo_Btn
			// 
			this.BasicInfo_Btn.Location = new System.Drawing.Point(608, 496);
			this.BasicInfo_Btn.Name = "BasicInfo_Btn";
			this.BasicInfo_Btn.Size = new System.Drawing.Size(128, 40);
			this.BasicInfo_Btn.TabIndex = 8;
			this.BasicInfo_Btn.Text = "See Basic Information";
			this.BasicInfo_Btn.Click += new System.EventHandler(this.BasicInfo_Btn_Click);
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(312, 16);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(208, 24);
			this.label6.TabIndex = 9;
			this.label6.Text = "Number of Sessions Required :";
			// 
			// NumOfSession_TB
			// 
			this.NumOfSession_TB.Location = new System.Drawing.Point(536, 8);
			this.NumOfSession_TB.Name = "NumOfSession_TB";
			this.NumOfSession_TB.Size = new System.Drawing.Size(136, 23);
			this.NumOfSession_TB.TabIndex = 10;
			this.NumOfSession_TB.Text = "7";
			// 
			// NumOfRound_Btn
			// 
			this.NumOfRound_Btn.Location = new System.Drawing.Point(312, 48);
			this.NumOfRound_Btn.Name = "NumOfRound_Btn";
			this.NumOfRound_Btn.Size = new System.Drawing.Size(272, 24);
			this.NumOfRound_Btn.TabIndex = 11;
			this.NumOfRound_Btn.Text = "Number of Rounds  for Each Session :";
			// 
			// NumOfRound_TB
			// 
			this.NumOfRound_TB.Location = new System.Drawing.Point(600, 40);
			this.NumOfRound_TB.Name = "NumOfRound_TB";
			this.NumOfRound_TB.Size = new System.Drawing.Size(128, 23);
			this.NumOfRound_TB.TabIndex = 12;
			this.NumOfRound_TB.Text = "3";
			// 
			// label7
			// 
			this.label7.Location = new System.Drawing.Point(320, 152);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(160, 16);
			this.label7.TabIndex = 13;
			this.label7.Text = "Session :";
			// 
			// label8
			// 
			this.label8.Location = new System.Drawing.Point(536, 152);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(200, 16);
			this.label8.TabIndex = 14;
			this.label8.Text = "Round for Each Session :";
			// 
			// listBox_Session
			// 
			this.listBox_Session.ItemHeight = 16;
			this.listBox_Session.Location = new System.Drawing.Point(320, 176);
			this.listBox_Session.Name = "listBox_Session";
			this.listBox_Session.ScrollAlwaysVisible = true;
			this.listBox_Session.Size = new System.Drawing.Size(192, 84);
			this.listBox_Session.TabIndex = 15;
			this.listBox_Session.SelectedIndexChanged += new System.EventHandler(this.listBox_Session_SelectedIndexChanged);
			// 
			// listBox_Round
			// 
			this.listBox_Round.ItemHeight = 16;
			this.listBox_Round.Location = new System.Drawing.Point(544, 176);
			this.listBox_Round.Name = "listBox_Round";
			this.listBox_Round.ScrollAlwaysVisible = true;
			this.listBox_Round.Size = new System.Drawing.Size(184, 84);
			this.listBox_Round.TabIndex = 16;
			this.listBox_Round.SelectedIndexChanged += new System.EventHandler(this.listBox_Round_SelectedIndexChanged);
			// 
			// Gen_Session_Round_Btn
			// 
			this.Gen_Session_Round_Btn.Location = new System.Drawing.Point(304, 88);
			this.Gen_Session_Round_Btn.Name = "Gen_Session_Round_Btn";
			this.Gen_Session_Round_Btn.Size = new System.Drawing.Size(224, 56);
			this.Gen_Session_Round_Btn.TabIndex = 17;
			this.Gen_Session_Round_Btn.Text = "Generate Session and Round Directories to Store Files to be Received.";
			this.Gen_Session_Round_Btn.Click += new System.EventHandler(this.Gen_Session_Round_Btn_Click);
			// 
			// Available_Files_TB
			// 
			this.Available_Files_TB.Location = new System.Drawing.Point(544, 312);
			this.Available_Files_TB.Multiline = true;
			this.Available_Files_TB.Name = "Available_Files_TB";
			this.Available_Files_TB.ReadOnly = true;
			this.Available_Files_TB.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.Available_Files_TB.Size = new System.Drawing.Size(192, 176);
			this.Available_Files_TB.TabIndex = 18;
			this.Available_Files_TB.Text = "";
			// 
			// AvailanleFiles_Btn
			// 
			this.AvailanleFiles_Btn.Location = new System.Drawing.Point(552, 272);
			this.AvailanleFiles_Btn.Name = "AvailanleFiles_Btn";
			this.AvailanleFiles_Btn.Size = new System.Drawing.Size(176, 32);
			this.AvailanleFiles_Btn.TabIndex = 20;
			this.AvailanleFiles_Btn.Text = "See Available Files";
			this.AvailanleFiles_Btn.Click += new System.EventHandler(this.AvailanleFiles_Btn_Click);
			// 
			// DeleteFile_Btn
			// 
			this.DeleteFile_Btn.Location = new System.Drawing.Point(536, 88);
			this.DeleteFile_Btn.Name = "DeleteFile_Btn";
			this.DeleteFile_Btn.Size = new System.Drawing.Size(192, 56);
			this.DeleteFile_Btn.TabIndex = 21;
			this.DeleteFile_Btn.Text = "Delete Files In the Selected Session and Round.";
			this.DeleteFile_Btn.Click += new System.EventHandler(this.DeleteFile_Btn_Click);
			// 
			// GraphForEachRound_Btn
			// 
			this.GraphForEachRound_Btn.Location = new System.Drawing.Point(464, 496);
			this.GraphForEachRound_Btn.Name = "GraphForEachRound_Btn";
			this.GraphForEachRound_Btn.Size = new System.Drawing.Size(136, 40);
			this.GraphForEachRound_Btn.TabIndex = 22;
			this.GraphForEachRound_Btn.Text = "Show Graph For Each Round";
			this.GraphForEachRound_Btn.Click += new System.EventHandler(this.GraphForEachRound_Btn_Click);
			// 
			// GraphAllRounds_Btn
			// 
			this.GraphAllRounds_Btn.Location = new System.Drawing.Point(312, 496);
			this.GraphAllRounds_Btn.Name = "GraphAllRounds_Btn";
			this.GraphAllRounds_Btn.Size = new System.Drawing.Size(144, 40);
			this.GraphAllRounds_Btn.TabIndex = 23;
			this.GraphAllRounds_Btn.Text = "Show Mean Graph For All Rounds";
			this.GraphAllRounds_Btn.Click += new System.EventHandler(this.GraphAllRounds_Btn_Click);
			// 
			// ResetForStatusTB_Btn
			// 
			this.ResetForStatusTB_Btn.Location = new System.Drawing.Point(320, 272);
			this.ResetForStatusTB_Btn.Name = "ResetForStatusTB_Btn";
			this.ResetForStatusTB_Btn.Size = new System.Drawing.Size(216, 40);
			this.ResetForStatusTB_Btn.TabIndex = 24;
			this.ResetForStatusTB_Btn.Text = "Reset For the Below Status In Receiving Files";
			this.ResetForStatusTB_Btn.Click += new System.EventHandler(this.ResetForStatusTB_Btn_Click);
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label5.Location = new System.Drawing.Point(24, 48);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(264, 40);
			this.label5.TabIndex = 14;
			this.label5.Text = "Designate the Place  You Want to Collect Reesult Files :";
			// 
			// Path_TB
			// 
			this.Path_TB.Location = new System.Drawing.Point(24, 96);
			this.Path_TB.Multiline = true;
			this.Path_TB.Name = "Path_TB";
			this.Path_TB.Size = new System.Drawing.Size(272, 56);
			this.Path_TB.TabIndex = 15;
			this.Path_TB.Text = "\\MyData\\Jin-Hee\\PocketPC2002Chat\\ResultsCollected\\";
			// 
			// StatisticalAnalyzer
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(7, 16);
			this.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(192)), ((System.Byte)(192)), ((System.Byte)(0)));
			this.ClientSize = new System.Drawing.Size(744, 541);
			this.Controls.Add(this.ResetForStatusTB_Btn);
			this.Controls.Add(this.GraphAllRounds_Btn);
			this.Controls.Add(this.GraphForEachRound_Btn);
			this.Controls.Add(this.DeleteFile_Btn);
			this.Controls.Add(this.AvailanleFiles_Btn);
			this.Controls.Add(this.Available_Files_TB);
			this.Controls.Add(this.Gen_Session_Round_Btn);
			this.Controls.Add(this.listBox_Round);
			this.Controls.Add(this.listBox_Session);
			this.Controls.Add(this.label8);
			this.Controls.Add(this.label7);
			this.Controls.Add(this.NumOfRound_TB);
			this.Controls.Add(this.NumOfRound_Btn);
			this.Controls.Add(this.NumOfSession_TB);
			this.Controls.Add(this.label6);
			this.Controls.Add(this.BasicInfo_Btn);
			this.Controls.Add(this.Status_TB);
			this.Controls.Add(this.panel_KeyParameter);
			this.Font = new System.Drawing.Font("Microsoft Sans Serif", 10F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "StatisticalAnalyzer";
			this.Text = "StatisticalAnalyzer";
			this.Load += new System.EventHandler(this.StatisticalAnalyzer_Load_1);
			this.panel_KeyParameter.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void SetXParameter()
		{
			try
			{
				XParam_LB.Items.Add("Speed");
				XParam_LB.Items.Add("Pause Time");
				XParam_LB.Items.Add("SWS");
				XParam_LB.Items.Add("Number of Nodes");
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SetXParameter(): "+e.Message);
			}
		}

		private void SetYParameter()
		{
			try
			{
                YParam_LB.Items.Add(COMMONCONSTANTS.AVE_LATENCY_FINDING_ROUTE);
	            YParam_LB.Items.Add(COMMONCONSTANTS.AVE_LATENCY_DELIVERYING_DP);
				YParam_LB.Items.Add(COMMONCONSTANTS.DELIVERY_RATIO_DP);
				YParam_LB.Items.Add(COMMONCONSTANTS.NORMALIZED_CONTROL_OVERHEAD);
				YParam_LB.Items.Add(COMMONCONSTANTS.THROUGHPUT);
				YParam_LB.Items.Add(COMMONCONSTANTS.NUM_OF_CACHE_ENTRIES);
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SetYParameter(): "+e.Message);
			}
		}

		private void StatisticalAnalyzer_Load_1(object sender, System.EventArgs e)
		{
			IPHostEntry localHostEntry = Dns.GetHostByName(Dns.GetHostName());
			IPAddress myIP = localHostEntry.AddressList[0];
			strMyIP = myIP.ToString();	
			MyIP.Text = strMyIP;
			strMyID = "SA";//Statistical Analyzer
			SetXParameter();
			SetYParameter();			
		}

		private void DivideIntoProperDataSize(string p_message)
		{
			try
			{
				int numOfTotalPackets = 0;
				int messageSize =  UDPCONSTANTS.MESSAGESIZE-300;
				int round  = 0;
				int wholeMSGLength = p_message.Length;
				double db_messageSize = messageSize;
				double db_wholeMSGLength = wholeMSGLength;
				int int_NOP = (wholeMSGLength/messageSize);
				double db_NOP = (db_wholeMSGLength/db_messageSize);
				if(db_NOP.CompareTo(double.Parse(int_NOP.ToString()))>0)
				{
					numOfTotalPackets = int_NOP+1;
				}
				else
				{
					numOfTotalPackets = int_NOP;				 
				}
				
				//MessageBox.Show("numOfTotalPackets: "+numOfTotalPackets);
				FilePacket initial_fp = new FilePacket();
				string temp_file_name = "GEN_basicInfo.xml";
				initial_fp = createFilePacket(temp_file_name,numOfTotalPackets.ToString());
				DataToSendBuffer.Add(initial_fp); 		
				while(messageSize*round<wholeMSGLength)
				{
					round++;
					string messageToSend = "";
					if(messageSize*round>wholeMSGLength)
					{
						messageToSend = p_message.Substring(messageSize*(round-1));
					}
					else
					{
						messageToSend = p_message.Substring(messageSize*(round-1),messageSize);
					}					
					FilePacket fp = new FilePacket();
					fp = createFilePacket(messageToSend,numOfTotalPackets.ToString());
					DataToSendBuffer.Add(fp); 			
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in DivideIntoProperDataSize(): "+e.Message);
			}
		}

		private string showDataToSendBuffer()
		{
			string inside = "DataToSendBuffer: ";
			try
			{
				for(int i =0; i<DataToSendBuffer.Count; i++)
				{
					FilePacket fp = (FilePacket)DataToSendBuffer[i];
					inside += fp.strID+" ";
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in showDataToSendBuffer() : "+e.Message);
			}
			return inside;
		}

		public void SendFilePacket(FilePacket fp)
		{
			try
			{
				XmlDocument doc = new XmlDocument();
				doc.LoadXml("<FilePacket></FilePacket>");

				XmlNode root = doc.DocumentElement;

				//1. ID
				XmlElement elem = doc.CreateElement(COMMONCONSTANTS.ID);
				elem.InnerText = fp.strID;
				root.AppendChild(elem);

				//2. SenderIP
				elem = doc.CreateElement(COMMONCONSTANTS.SENDERIP);
				elem.InnerText = fp.strSenderIP;
				root.AppendChild(elem);

				//3. Number of Total Packets
				elem = doc.CreateElement(COMMONCONSTANTS.NUMOFTOTALPACKETS);
				elem.InnerText = fp.strNumOfTotalPackets;
				root.AppendChild(elem);

				//4. File Content
				elem = doc.CreateElement(COMMONCONSTANTS.FILECONTENT);
				elem.InnerText = fp.strXMLFile;
				root.AppendChild(elem);

				//5. Type
				elem = doc.CreateElement(COMMONCONSTANTS.TYPE);
				elem.InnerText = fp.strType;
				root.AppendChild(elem);

				string messageToSend = doc.InnerXml.ToString();
				
				SetReceiptTimer(fp);
				sendUDP(strMyIP, messageToSend);
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SendFilePacket(): "+e.Message);
			}
		}

		private void SaveInSenderBuffer(FilePacket p_fp)
		{
			SenderBuffer.Add(p_fp.strID, p_fp);
		}

		private string getID()
		{
			fileID++;
			return fileID.ToString();
		}

		private FilePacket createFilePacket(string p_messageToSend, string p_numOfTotalPackets)
		{
			FilePacket fp = new FilePacket();
			try
			{                
				fp.strType = COMMONCONSTANTS.FILE_PACKET;
				fp.strID = strMyID+":"+getID();
				fp.strNumOfTotalPackets = p_numOfTotalPackets;
				fp.strSenderIP = strMyIP;
				fp.strXMLFile = p_messageToSend;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in createFilePacket(): "+e.Message);
			}
			return fp;
		}


		public void ReleaseReceiptTimer(string p_ID)
		{
			try
			{
				if(ReceiptTimerBuffer.Contains(p_ID))
				{
					IDictionaryEnumerator ide = ReceiptTimerBuffer.GetEnumerator();
					while(ide.MoveNext())
					{
						if(ide.Key.ToString().Equals(p_ID))
						{
							MyTimer myTimer = (MyTimer)ide.Value;
							myTimer.setEnabled(false);
							break;
						}
					}
				}
				else
				{
					MessageBox.Show("Not "+p_ID+" in ReceiptTimerBuffer");
				}
				if(ReceiptTimerBuffer.Contains(p_ID))
				{
					ReceiptTimerBuffer.Remove(p_ID);
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in ReleaseReceiptTimer(): "+e.Message+" ID: "+p_ID);
			}
		}

		private void DeleteAllDirectories()
		{
			try
			{				
				int numOfRound = int.Parse(NumOfRound_TB.Text);
				int numOfSession = int.Parse(NumOfSession_TB.Text);
				for(int i = 1; i<=numOfSession; i++)
				{										
					for(int j = 1 ; j<=numOfRound; j++)
					{						
						string path =Path_TB.Text+"Session_"+i.ToString()+"\\Round_"+j.ToString();					
						if(Directory.Exists(@path))
						{
							Directory.Delete(@path);
						}
					}
					string session_path =Path_TB.Text+"Session_"+i.ToString();					
					if(Directory.Exists(@session_path))
					{
						Directory.Delete(@session_path);
					}
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in DeleteAllDirectories(): "+e.Message);
			}
		}
		private void Exit_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{
			/*	DeleteAllFiles();
				if(!NumOfRound_TB.Text.Equals("") && !NumOfSession_TB.Text.Equals(""))
				{
					DeleteAllDirectories();
				}*/
				udpServerIsAlive = false;
				stop_server();			
				resetAll();
				this.Close();
			}
			catch(Exception ex)
			{
                MessageBox.Show("Exception in Exit_Btn_Click(): "+ex.Message);
			}
		}

		private Hashtable getADCInfo(string p_path)
		{
            Hashtable ADCBasicInfoBuffer = new Hashtable();
			try
			{
				StreamReader conf = new StreamReader(@p_path);
				string strConfXML = conf.ReadToEnd();
				conf.Close();		
				XmlDocument xmlDoc = new XmlDocument();				
				xmlDoc.LoadXml(strConfXML);
				XmlNode root = xmlDoc.DocumentElement;//<Configuration>
				XmlNodeList childNodes = root.ChildNodes;//<Node>
				foreach(XmlNode childNode in childNodes)
				{					
					XmlElement currentElement = (XmlElement)childNode;
					if(currentElement.Name.Equals("Source"))
					{
						ADCBasicInfoBuffer.Add("Source",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("Target"))
					{
						ADCBasicInfoBuffer.Add("Target",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("ACM"))
					{
						ADCBasicInfoBuffer.Add("ACM",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("Cache"))
					{
						ADCBasicInfoBuffer.Add("Cache",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("SWS"))
					{
						ADCBasicInfoBuffer.Add("SWS",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("NumOfDataPacket"))
					{
						ADCBasicInfoBuffer.Add("NumOfDataPacket",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("ReceiptPacket"))
					{
						ADCBasicInfoBuffer.Add("ReceiptPacket",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("TrafficControlParam"))
					{
						ADCBasicInfoBuffer.Add("TrafficControlParam",currentElement.FirstChild.Value);
					}
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getADCInfo(): "+e.Message);
			}
			return ADCBasicInfoBuffer;		
		}
		private Hashtable getGENInfo(string p_path)
		{
			Hashtable GENBasicInfoBuffer = new Hashtable();
			try
			{
				StreamReader conf = new StreamReader(@p_path);
				string strConfXML = conf.ReadToEnd();
				conf.Close();		
				XmlDocument xmlDoc = new XmlDocument();				
				xmlDoc.LoadXml(strConfXML);
				XmlNode root = xmlDoc.DocumentElement;
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{					
					XmlElement currentElement = (XmlElement)childNode;
					if(currentElement.Name.Equals("NumOfNodes"))
					{
						GENBasicInfoBuffer.Add("NumOfNodes",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("SimulationArea"))
					{
						XmlNodeList grandChildNodeList = currentElement.ChildNodes;
						foreach(XmlNode grandchildNode in grandChildNodeList)
						{
							XmlElement xNode = (XmlElement)grandchildNode;
							if(xNode.Name.Equals("Width"))
							{
								GENBasicInfoBuffer.Add("SA_Width", xNode.FirstChild.Value);
							}
							else//Height
							{
								GENBasicInfoBuffer.Add("SA_Height", xNode.FirstChild.Value);                         
							}
						}
					}
					else if(currentElement.Name.Equals("PowerRange"))
					{
						GENBasicInfoBuffer.Add("PowerRange",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("Speed"))
					{
						GENBasicInfoBuffer.Add("Speed",currentElement.FirstChild.Value);
					}
					else if(currentElement.Name.Equals("PauseTime"))
					{
						GENBasicInfoBuffer.Add("PauseTime",currentElement.FirstChild.Value);
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getGENInfo(): "+e.Message);
			}
			return GENBasicInfoBuffer;
		}

		private void BasicInfo_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{
				string GEN_path = Path_TB.Text+
					selectedSession+"\\"+selectedRound+"\\GEN_basicInfo.xml";
				string ADC_path =  Path_TB.Text+
					selectedSession+"\\"+selectedRound+"\\ADC_basicInfo.xml";					
				if((File.Exists(@GEN_path)) && (File.Exists(@ADC_path)))
				{
					Hashtable GENInfo = getGENInfo(GEN_path);
					Hashtable ADCInfo = getADCInfo(ADC_path);
				
					BasicInfo bi = new BasicInfo(ADCInfo, GENInfo);
					bi.Top = 0;
					bi.Focus();
					bi.BringToFront();
					bi.Visible = true;
				}
				else
				{
                    MessageBox.Show("No proper data available to show basic information!"); 
				}
			}
			catch(Exception ex)
			{
               MessageBox.Show("Exception in BasicInfo_Btn_Click(): "+ex.Message);
			}
		}

		private void SetListBoxSession()
		{
			try
			{				
				int numOfSession = int.Parse(NumOfSession_TB.Text);
				for(int i = 1; i<=numOfSession; i++)
				{
					listBox_Session.Items.Add("Session_"+i.ToString());
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in SetListBoxSession(): "+e.Message);
			}
	    }

		private void SetListBoxRound()
		{
			try
			{
			    int numOfRound = int.Parse(NumOfRound_TB.Text);
				for(int i = 1; i<=numOfRound; i++)
				{
					listBox_Round.Items.Add("Round_"+i.ToString());
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SetListBoxRound(): "+e.Message);
			}
		}
		
		private void generateSessionDirectory()
		{
			try
			{
				int numOfSession = int.Parse(NumOfSession_TB.Text);
				for(int i = 1; i<=numOfSession; i++)
				{
						Directory.CreateDirectory(
							@Path_TB.Text+"Session_"+i.ToString());
				}

			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in generateSessionDirectory(): "+e.Message);   
			}
		}
 
		private void generateRoundDirectory()
		{
			try
			{
				int numOfRound = int.Parse(NumOfRound_TB.Text);
				int numOfSession = int.Parse(NumOfSession_TB.Text);
				for(int i = 1; i<=numOfSession; i++)
				{					
					for(int j = 1 ; j<=numOfRound; j++)
					{						
						string path =Path_TB.Text+"Session_"+i.ToString()+"\\Round_"+j.ToString();					
						Directory.CreateDirectory(@path);							
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in generateRoundDirectory(): "+e.Message);   
			}           
		}

		private void Gen_Session_Round_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{			
				listBox_Session.Items.Clear();
				listBox_Round.Items.Clear();
				int numOfRound = int.Parse(NumOfRound_TB.Text);
				int numOfSession = int.Parse(NumOfSession_TB.Text);				
				SetListBoxSession();
				SetListBoxRound();				
				generateSessionDirectory();
				generateRoundDirectory();
			}
			catch(FormatException fe)
			{
				MessageBox.Show("Please enter an number format of inputs! : "+fe.Message);
			}
			catch(Exception ex)
			{
                MessageBox.Show("Exception in Gen_Session_Round_Btn_Click(): "+ex.Message);     
			}
		}

		private void SetPossibleFilesToBeReceived(string p_strConfXML)
		{
			try
			{
				possibleFilesToBeReceived.Add("conf.xml");
				possibleFilesToBeReceived.Add("GEN_basicInfo.xml");
				possibleFilesToBeReceived.Add("ADC_basicInfo.xml");
				XmlDocument xmlDoc = new XmlDocument();				
				xmlDoc.LoadXml(p_strConfXML);
				XmlNode root = xmlDoc.DocumentElement;//<Configuration>
				XmlNodeList childNodes = root.ChildNodes;//<Node>
				foreach(XmlNode childNode in childNodes)
				{
					foreach(XmlAttribute a in childNode.Attributes)
					{
						if(a.Name.Equals("ID"))
						{
							string tempFile_Result = "Result_"+a.Value+".xml";
							string tempFile_DA = "DataAnalysis_"+a.Value+".xml";
							possibleFilesToBeReceived.Add(tempFile_Result);
							possibleFilesToBeReceived.Add(tempFile_DA);
							break;
						}
					}
				}
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in SetPossibleFilesToBeReceived(): "+e.Message);
			}
		}

		private void AvailanleFiles_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{				
				if(selectedSession.Equals("") || selectedRound.Equals(""))
				{
                    MessageBox.Show("Please select the session and round you wish!");
				}
				else
				{
					possibleFilesToBeReceived.Clear();
					string availableFiles = "";
					string path = Path_TB.Text+selectedSession+
						"\\"+selectedRound+"\\conf.xml";				
					if(File.Exists(@path))
					{
						StreamReader conf = new StreamReader(@path);
						string strConfXML = conf.ReadToEnd();
						conf.Close();				
						SetPossibleFilesToBeReceived(strConfXML);

						string commonPath = Path_TB.Text+selectedSession+
							"\\"+selectedRound+"\\";
						for(int i = 0 ; i<possibleFilesToBeReceived.Count; i++)
						{
							string possibleFile = possibleFilesToBeReceived[i].ToString();
							if(File.Exists(@commonPath+possibleFile))
							{
								availableFiles += possibleFile+"\r\n";
							}
						}
						Available_Files_TB.Text = availableFiles;
					}
					else
					{
						Available_Files_TB.Text ="Due to no existence of the conf.xml, no files are detected!";
					}
				}
			}
			catch(Exception ex)
			{
               MessageBox.Show("Exception in AvailanleFiles_Btn_Click(): "+ex.Message);
			}
		}

		private void DeleteFile_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{
				possibleFilesToBeReceived.Clear();
				string availableFiles = "";
				string path = Path_TB.Text+selectedSession+
					"\\"+selectedRound+"\\conf.xml";
				if(File.Exists(@path))
				{
					StreamReader conf = new StreamReader(@path);
					string strConfXML = conf.ReadToEnd();
					conf.Close();				
					SetPossibleFilesToBeReceived(strConfXML);

					string commonPath = Path_TB.Text+selectedSession+
						"\\"+selectedRound+"\\";
				
					for(int i = 0 ; i<possibleFilesToBeReceived.Count; i++)
					{
						string possibleFile = commonPath+possibleFilesToBeReceived[i].ToString();
						if(File.Exists(@possibleFile))
						{
							File.Delete(@possibleFile);
						}
					}
				}
			}
			catch(Exception ex)
			{
                MessageBox.Show("Exception in DeleteFile_Btn_Click(): "+ex.Message);
			}
		}


		private void DeleteFiles_Btn_Click(object sender, System.EventArgs e)
		{
			try
			{
				int numOfSession = int.Parse(NumOfSession_TB.Text);
				int numOfRound = int.Parse(NumOfRound_TB.Text);
				string commonPath = Path_TB.Text;
				for(int i = 1; i<= numOfSession; i++)
				{
					string sessionPath = commonPath+"Session_"+i.ToString();
					for(int j = 1; j<= numOfRound; j++)
					{
                        string roundPath = sessionPath+"\\Round_"+j.ToString();
						possibleFilesToBeReceived.Clear();
						string path = roundPath+"\\conf.xml";
						if(File.Exists(@path))
						{
							StreamReader conf = new StreamReader(@path);
							string strConfXML = conf.ReadToEnd();
							conf.Close();				
							SetPossibleFilesToBeReceived(strConfXML);
							for(int k =0; k<possibleFilesToBeReceived.Count; k++)
							{
								string filePath = roundPath+"\\"+possibleFilesToBeReceived[k].ToString();
								if(File.Exists(@filePath))
								{
									File.Delete(@filePath); 
								}
							}//file for
						 }
						if(Directory.Exists(@roundPath))
						{
							//MessageBox.Show("roundPath: "+roundPath);
							Directory.Delete(@roundPath);
						}
					 }//round inside
					if(Directory.Exists(@sessionPath))
					{
						//MessageBox.Show("sessionPath: "+sessionPath);
						Directory.Delete(@sessionPath);
					}
				}//session inside
			}		
			catch(Exception ex)
			{
               MessageBox.Show("Exception in DeleteFiles_Btn_Click(): "+ex.Message);
			}
		}

		private void listBox_Session_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			try
			{
				selectedSession = listBox_Session.SelectedItem.ToString();
			}
			catch(Exception ex)
			{
               MessageBox.Show("Exception in listBox_Session_SelectedIndexChanged(): "+ex.Message);
			}
		}

		private void listBox_Round_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			try
			{
    			selectedRound = listBox_Round.SelectedItem.ToString();		
			}
			catch(Exception ex)
			{
				MessageBox.Show("Exception in listBox_Round_SelectedIndexChanged(): "+ex.Message);
			}
		}

		private Hashtable getProperDataForYParams()
		{
			Hashtable DataForEachRound = new Hashtable();
			try
			{
				int tempNumOfRound = int.Parse(NumOfRound_TB.Text);
				for(int i = 1; i<=tempNumOfRound; i++)
				{
					string strRound = "Round_"+i.ToString();
					ArrayList YValues = getProperYValues(strRound);
					DataForEachRound.Add(i.ToString(), YValues);
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getProperDataForYParams(): "+e.Message);
			}
			return DataForEachRound;
		}

		private string getAvgLatencyForFindingRouteFromXML(string p_path)
		{
			string L = "N/A";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.ADFR))
					{
						L = elem.FirstChild.Value;
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getAvgLatencyForFindingRouteFromXML(): "+e.Message);
			}
			return L;
		}
		private ArrayList getAvgLatencyForFindingRoute(string p_strRound)
		{
			try
			{
				ArrayList AvgLatencyForFindingRoute = new ArrayList();
				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string commonPath = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;
					string sourceNode = getSourceNode(commonPath);
					string source_file = "DataAnalysis_"+sourceNode+".xml";
					string path = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
                    string AvgLatency = getAvgLatencyForFindingRouteFromXML(path);
					AvgLatencyForFindingRoute.Add(AvgLatency);
				}
				return AvgLatencyForFindingRoute;
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in getAvgLatencyForRepairingRoute(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private ArrayList getPaticipatingNodes(string p_path)
		{
			ArrayList neighbors = new ArrayList();
			try
			{
				XmlDocument xmlDoc = new XmlDocument();
				string path = p_path+"\\conf.xml";
				xmlDoc.Load(path);
				XmlNode root = xmlDoc.DocumentElement;//<Configuration>
				XmlNodeList xNodes = root.ChildNodes;
				foreach(XmlNode childNode in xNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					foreach(XmlAttribute a in elem.Attributes)
					{
						if(a.Name.Equals("ID"))
						{							
                            neighbors.Add(a.Value);
							break;
						}
					}
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getPaticipatingNodes(): "+e.Message);
			}
			return neighbors;
		}

		private string getAvgLatencyForDeliveryingDPFromXML(string p_path)
		{
			string L = "N/A";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.ALDDP))
					{
						L = elem.FirstChild.Value;
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getAvgLatencyForDeliveryingDPFromXML(): "+e.Message);
			}
			return L;
		}
		private ArrayList getAvgLatencyForDeliveryingDP(string p_strRound)
		{
			try
			{
				ArrayList AvgLatencyForDeliveryingDP = new ArrayList();

				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string path = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;
					string sourceNode = getSourceNode(path);
					string source_file = "DataAnalysis_"+sourceNode+".xml";
					string pathToSource = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
					string avgLatency = getAvgLatencyForDeliveryingDPFromXML(pathToSource);
                    AvgLatencyForDeliveryingDP.Add(avgLatency);
				}
				return AvgLatencyForDeliveryingDP;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getAvgLatencyForDeliveryingDP(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private double getTotalDPDeliveredFromXML(string p_path)
		{
			string DPDelivered = "N/A";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.TDPD))
					{
						DPDelivered = elem.FirstChild.Value;
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getTotalDPDeliveredFromXML(): "+e.Message);
			}
			double doubleDPDelivered = double.Parse(DPDelivered);
			return doubleDPDelivered;
		}

		private double getTotalNumOfControlPacketsFromXML(string p_commonPath)
		{
			string NumOfCP = "N/A";
			double sumOfCPSent = 0;
			try
			{
				ArrayList PNodes = getPaticipatingNodes(p_commonPath);
				IEnumerator ie = PNodes.GetEnumerator();
				while(ie.MoveNext())
				{
					string path = p_commonPath+"\\"+"DataAnalysis_"+ie.Current.ToString()+".xml";
					StreamReader myReader = new StreamReader(path,Encoding.ASCII);
					string fileContent = myReader.ReadToEnd();
					XmlDocument xmlDoc = new XmlDocument();
					xmlDoc.LoadXml(fileContent);
					XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
					XmlNodeList childNodes = root.ChildNodes;
					foreach(XmlNode childNode in childNodes)
					{
						XmlElement elem = (XmlElement)childNode;
						if(elem.Name.Equals(COMMONCONSTANTS.TNCPSent))
						{
							NumOfCP = elem.FirstChild.Value;
							break;
						}
					}
					double doubleNumOfCP = double.Parse(NumOfCP);
					sumOfCPSent += doubleNumOfCP;
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getTotalNumOfControlPacketsFromXML(): "+e.Message);
			}
			return sumOfCPSent;
		}
		private ArrayList getNormalizedControlOverhead(string p_strRound)
		{
			try
			{
				ArrayList NormalizedControlOverhead = new ArrayList();
				
				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string commonPath = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;				
					string sourceNode = getSourceNode(commonPath);
					string source_file = "DataAnalysis_"+sourceNode+".xml";
					string path = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
					double totalDPDelivered = getTotalDPDeliveredFromXML(path);					
					double totalControlPacketsUsed = getTotalNumOfControlPacketsFromXML(commonPath);
					double normalizedCO = (totalControlPacketsUsed/totalDPDelivered);
					NormalizedControlOverhead.Add(normalizedCO.ToString());
				}

				return NormalizedControlOverhead;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getNormalizedControlOverhead(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private string getDeliveryRatioFromXML(string p_path)
		{
			string DR = "N/A";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.DRDP))
					{
						DR = elem.FirstChild.Value;
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getDeliveryRatioFromXML(): "+e.Message);
			}
			return DR;
		}
		private ArrayList getDeliveryRatioForDP(string p_strRound)
		{
			try
			{
				ArrayList DeliveryRatioForDP = new ArrayList();
				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string commonPath = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;
					string sourceNode = getSourceNode(commonPath);
					string source_file = "DataAnalysis_"+sourceNode+".xml";
					string path = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
					string DRDP = getDeliveryRatioFromXML(path);
					DeliveryRatioForDP.Add(DRDP);
				}
				return DeliveryRatioForDP;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getDeliveryRatioForDP(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private string getSourceID(string p_source)
		{
			string sourceID ="N/A";
			try
			{
                int index = p_source.IndexOf(":");
				sourceID = p_source.Substring(0, index);
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getSourceID(): "+e.Message);
			}
			return sourceID;
		}

		private string getSourceNode(string p_path)
		{
            string sourceNode = "NA";  
			try
			{

				string path = p_path +"\\ADC_BasicInfo.xml";
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.Load(path);
				XmlNode root = xmlDoc.DocumentElement;
				XmlNodeList xNodes = root.ChildNodes;
				foreach(XmlNode childNode in xNodes)
				{					
					XmlElement currentElement = (XmlElement)childNode;
					if(currentElement.Name.Equals("Source"))
					{
                        string temp = currentElement.FirstChild.Value;
						sourceNode = getSourceID(temp);
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getSourceNode(): "+e.Message);
			}
			return sourceNode;
		}
		private string getAvgNumOfCacheEntriesFromXML(string p_path)
		{
			string CE = "N/A";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.AVG_CACHE_ENTRIES))
					{
						CE = elem.FirstChild.Value;
						//MessageBox.Show(CE);
						break;
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getThroughputFromXML(): "+e.Message);
			}
			return CE;
		}
		private string getThroughputFromXML(string p_path)
		{
			string TP = "N/A";
			try
			{
                StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;//<DataAnalysis>
				XmlNodeList childNodes = root.ChildNodes;
				foreach(XmlNode childNode in childNodes)
				{
					XmlElement elem = (XmlElement)childNode;
					if(elem.Name.Equals(COMMONCONSTANTS.THROUGHPUT))
					{
                        TP = elem.FirstChild.Value;
						break;
					}
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getThroughputFromXML(): "+e.Message);
			}
			return TP;
		}

		private ArrayList getAvgNumOfCacheEntries(string p_strRound)
		{
			try
			{
				ArrayList NumOfCacheEntries = new ArrayList();

				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string path =Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;
					string sourceID = getSourceNode(path);
					string source_file = "DataAnalysis_"+sourceID+".xml";
					string pathToSource = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
					string  EachCE= getAvgNumOfCacheEntriesFromXML(pathToSource);
					NumOfCacheEntries.Add(EachCE);
				}
				return NumOfCacheEntries;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getAveNumOfCacheEntries(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}		
		}
		private ArrayList getThroughput(string p_strRound)
		{
			try
			{
				ArrayList Throughput = new ArrayList();

				int tempNumOfSession = int.Parse(NumOfSession_TB.Text);				
				for(int i = 1; i<=tempNumOfSession; i++)
				{
					string path =Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound;
					string sourceID = getSourceNode(path);
					string source_file = "DataAnalysis_"+sourceID+".xml";
					string pathToSource = Path_TB.Text+"Session_"+
						i.ToString()+"\\"+p_strRound+"\\"+source_file;
					string throughput = getThroughputFromXML(pathToSource);
					Throughput.Add(throughput);
				}
				return Throughput;
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getThroughput(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private ArrayList getProperYValues(string p_strRound)
		{
			try
			{
				ArrayList properYValue = new ArrayList();
                string tempYParam = selectedYParam;
				if(selectedYParam.Equals(COMMONCONSTANTS.AVE_LATENCY_FINDING_ROUTE))
				{
					properYValue = getAvgLatencyForFindingRoute(p_strRound);
				}
				else if(selectedYParam.Equals(COMMONCONSTANTS.AVE_LATENCY_DELIVERYING_DP))
				{
					properYValue = getAvgLatencyForDeliveryingDP(p_strRound);
				}
				else if(selectedYParam.Equals(COMMONCONSTANTS.NORMALIZED_CONTROL_OVERHEAD))
				{
					properYValue = getNormalizedControlOverhead(p_strRound);
				}
				else if(selectedYParam.Equals(COMMONCONSTANTS.DELIVERY_RATIO_DP))
				{
					properYValue = getDeliveryRatioForDP(p_strRound);
				}
				else if(selectedYParam.Equals(COMMONCONSTANTS.THROUGHPUT))
				{
					properYValue = getThroughput(p_strRound);
				}
				else if(selectedYParam.Equals(COMMONCONSTANTS.NUM_OF_CACHE_ENTRIES))
				{
                    properYValue = getAvgNumOfCacheEntries(p_strRound);
				}
				return properYValue;
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in getProperYValue(): "+e.Message);
				ArrayList temp = new ArrayList();
				return temp;
			}
		}

		private string getNumOfNodes(string p_path)
		{
			string numOfNodes = "";
			try
			{
				StreamReader myReader = new StreamReader(p_path,Encoding.ASCII);
				string fileContent = myReader.ReadToEnd();
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.LoadXml(fileContent);
				XmlNode root = xmlDoc.DocumentElement;
				XmlNodeList xNodes = root.ChildNodes;
				foreach(XmlNode childNode in xNodes)
				{			
					if(childNode.Name.Equals("NumOfNodes"))
					{
                        numOfNodes = childNode.FirstChild.Value;
						break;
					}
				}
			  
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in getNumOfNodes(): "+e.Message);
			}
			return numOfNodes;
		}
		private Hashtable getXParamIntervalFromXML()
		{
            Hashtable XParamInterval = new Hashtable();
			try
			{
				int NumOfSession = int.Parse(NumOfSession_TB.Text);		
				for(int i = 1; i<= NumOfSession; i++)
				{
					string path = Path_TB.Text+
						"Session_"+i.ToString()+"\\Round_1";
					if(selectedXParam.Equals("Number of Nodes"))
					{                        
						string tempPath = path +"\\GEN_basicInfo.xml";
						string numOfNodes = getNumOfNodes(tempPath);
						XParamInterval.Add(i.ToString(), numOfNodes);
					}
					else
					{
						string sourceNode = getSourceNode(path);
						string sourcePath = path+"\\DataAnalysis_"+sourceNode+".xml";
						StreamReader myReader = new StreamReader(sourcePath,Encoding.ASCII);
						string fileContent = myReader.ReadToEnd();
						XmlDocument xmlDoc = new XmlDocument();
						xmlDoc.LoadXml(fileContent);
						XmlNode root = xmlDoc.DocumentElement;
						XmlNodeList xNodes = root.ChildNodes;
						foreach(XmlNode childNode in xNodes)
						{					
							XmlElement currentElement = (XmlElement)childNode;
							if(currentElement.Name.Equals("XParam"))
							{
								XmlNodeList grandChildNodes = childNode.ChildNodes;
								foreach(XmlNode grandChildNode in grandChildNodes)
								{
									string temp = getXParam();
									if(grandChildNode.Name.Equals(temp))
									{                                   
										string tempXParam = grandChildNode.FirstChild.Value;
										if(!tempXParam.Equals("Random") && 
											!XParamInterval.ContainsValue(tempXParam))
										{
											XParamInterval.Add(i.ToString(),tempXParam);
										}
										break;
									}
								}							
								break;
							}
						}	
					}//else
				}//for
			}//try
			catch(Exception e)
			{
               MessageBox.Show("Exception in getXParamIntervalFromXML(): "+e.Message);
			}
			return XParamInterval;
		}

		private string getXParam()
		{
			string XParam = "";
			try
			{
				if(selectedXParam.Equals("Speed"))
				{
					XParam = "Speed";
				}
				else if(selectedXParam.Equals("SWS"))
				{
					XParam = "SWS";
				}
				else if(selectedXParam.Equals("Pause Time"))
				{
					XParam = "PauseTime";
				}
				else if(selectedXParam.Equals("Number of Nodes"))
				{
					XParam = "NumOfNodes";
				}
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in getXParam(): "+e.Message);
			}
            return XParam;
		}

		private void ShowHashInside(Hashtable p_hash)
		{
			try
			{
				string inside = "";
				IDictionaryEnumerator ide = p_hash.GetEnumerator();
				while(ide.MoveNext())
				{
					inside += ide.Key.ToString()+" "+ide.Value.ToString()+"\r\n";               
				}
				MessageBox.Show(inside);
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in ShowHashInside(): "+e.Message); 
			}
		}

		private void ShowHashYParam(Hashtable p_hash)
		{
			try
			{
                string inside = "";
				IDictionaryEnumerator ide = p_hash.GetEnumerator();
				while(ide.MoveNext())
				{
				    ArrayList al = (ArrayList)ide.Value;
					string strAl = "";
					for(int i =0; i<al.Count; i++)
					{
                       strAl += al[i].ToString()+" ";
					}
                   inside += ide.Key.ToString()+" "+strAl+"\r\n";
				}
				MessageBox.Show(inside);
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in ShowHashYParam(): "+e.Message);
			}
		}
		private GraphData GetMyGraphData()
		{			
			try
			{
				string x_param = selectedXParam;
				string y_param = selectedYParam;
				Hashtable XParam_Interval = getXParamIntervalFromXML();
				//Debugging
                //ShowHashInside(XParam_Interval);
				if(XParam_Interval.Count < 2)
				{
                    MessageBox.Show("Your selected X parameter is Not valid!");
					return new GraphData();
				}
				else
				{
					Hashtable DataForEachRound = getProperDataForYParams();
					//ShowHashYParam(DataForEachRound);
					GraphData myGD = new GraphData(x_param, 
						y_param, 
						XParam_Interval,
						DataForEachRound);
					return myGD;
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in GetMyGraphData(): "+e.Message);
			}
			return new GraphData();
		}

		private void GraphAllRounds_Btn_Click(object sender, System.EventArgs e)
		{
			if(!(selectedXParam.Equals("") || selectedYParam.Equals("")))
			{
				GraphData GD = GetMyGraphData();
				if(GD!=null)
				{
					MeanGraphForAllRounds mgfar = new MeanGraphForAllRounds(GD);
					mgfar.Top = 0;
					mgfar.Focus();
					mgfar.BringToFront();
					mgfar.Visible = true;
				}
			}
			else
			{
                MessageBox.Show("Please select Key Parameters for X-axis and Y-axis");
			}
		}

		private void GraphForEachRound_Btn_Click(object sender, System.EventArgs e)
		{
			if(!(selectedXParam.Equals("") || selectedYParam.Equals("")))
			{
				GraphData GD = GetMyGraphData();
				if(GD!=null)
				{
					GraphForEachRound gfer = new GraphForEachRound(GD);
					gfer.Top = 0;
					gfer.Visible = true;
					gfer.Focus();
					gfer.BringToFront();
				}
			}
			else
			{
				MessageBox.Show("Please select Key Parameters for X-axis and Y-axis");
			}
		}

		private void XParam_LB_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			selectedXParam = XParam_LB.SelectedItem.ToString();
		}

		private void YParam_LB_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			selectedYParam = YParam_LB.SelectedItem.ToString();		
		}

		private void ResetForStatusTB_Btn_Click(object sender, System.EventArgs e)
		{
			Status_TB.Text = "";
		}

		private void CompareGraph_Btn_Click(object sender, System.EventArgs e)
		{
			ComparisonOfGraphs CG = new ComparisonOfGraphs();
			CG.Top = 0;
			CG.Visible = true;
			CG.Focus();
			CG.BringToFront();
		}

		


	}//end of main()

	public struct SavedGraphElements
	{
        public string strCondition;
		public GraphData GD;
	}
	public struct FilePacket
	{
		public string strType;
		public string strNumOfTotalPackets;
        public string strSenderIP;
		public string strID;
		public string strXMLFile;
	}

	public class GraphData
	{
        public string x_parameter;
		public string y_parameter;
		public Hashtable XParam_Interval;
		public Hashtable DataForEachRound;

		public GraphData()
		{
		}

		public GraphData(string p_x_parameter, 
			string p_y_parameter, 
			Hashtable p_XParam_Interval,
			Hashtable p_DataForEachRound)
		{
			try
			{
                x_parameter = p_x_parameter;
				y_parameter = p_y_parameter;				
				XParam_Interval = p_XParam_Interval;
				DataForEachRound = p_DataForEachRound;
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in Constructor of GraphData(): "+e.Message);
			}
		}

	}
	public class MyTimer 
	{
		//properties	
		private Thread thread;
		private int _interval;
		private bool _enabled;
		private StatisticalAnalyzer _sa;
	    private string _ID;
		private FilePacket _fp;
		
		public MyTimer(int p_interval, bool p_enabled, 
			StatisticalAnalyzer p_sa, string p_ID,FilePacket p_fp)
		{
			_interval = p_interval;
			_enabled = p_enabled;
			_sa = p_sa;
			_fp = p_fp;
			_ID = p_ID;
		}

		public void setEnabled(bool p_enabled)
		{
			_enabled = p_enabled;				 
		}

		public void setInterval(int p_interval)
		{
			_interval = p_interval;
		}

		public int getInterval()
		{
			return _interval;
		}

		public bool getEnabled()
		{
			return _enabled;
		}

		public void SetReceiptTimer()
		{			
			thread = new Thread(new ThreadStart(this.ReceiptTimer_Click));
			thread.Start();
		}

		private void ReceiptTimer_Click()
		{
			try
			{				
				while(_enabled)
				{				
					Monitor.Enter(this);
					Thread.Sleep(_interval);
					if(_enabled)
					{						
						_sa.ReleaseReceiptTimer(_ID);
						if(_sa.SenderBuffer.Contains(_ID))
						{
							_sa.SendFilePacket(_fp);
						}
						break;
					}												
					Monitor.Exit(this);
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("ReceiptTimer_Click(): "+e.Message);
			}
		}


		
	}//inner class
}
